@@ -33,6 +33,8 @@
 #include <KSharedConfig>
 #include <KSystemClipboard>
 
+using namespace Qt::StringLiterals;
+
 ExportManager::ExportManager(QObject *parent)
     : QObject(parent)
     , m_imageSavedNotInTemp(false)
@@ -41,18 +43,15 @@ ExportManager::ExportManager(QObject *parent)
 {
     connect(this, &ExportManager::imageExported, [](Actions actions, const QUrl &url) {
         if (actions & AnySave) {
-            Settings::setLastSaveLocation(url);
+            Settings::setLastImageSaveLocation(url);
             if (actions & SaveAs) {
-                Settings::setLastSaveAsLocation(url);
+                Settings::setLastImageSaveAsLocation(url);
             }
         }
     });
 }
 
-ExportManager::~ExportManager()
-{
-    delete m_tempDir;
-}
+ExportManager::~ExportManager() = default;
 
 ExportManager *ExportManager::instance()
 {
@@ -116,19 +115,19 @@ static QString ensureDefaultLocationExists(const QUrl &saveUrl)
 
     QDir savePathDir(savePath);
     if (!(savePathDir.exists())) {
-        savePathDir.mkpath(QStringLiteral("."));
+        savePathDir.mkpath(u"."_s);
     }
     return savePath;
 }
 
 QString ExportManager::defaultSaveLocation() const
 {
-    return ensureDefaultLocationExists(Settings::self()->defaultSaveLocation());
+    return ensureDefaultLocationExists(Settings::imageSaveLocation());
 }
 
 QString ExportManager::defaultVideoSaveLocation() const
 {
-    return ensureDefaultLocationExists(Settings::self()->defaultVideoSaveLocation());
+    return ensureDefaultLocationExists(Settings::videoSaveLocation());
 }
 
 QUrl ExportManager::getAutosaveFilename() const
@@ -136,8 +135,9 @@ QUrl ExportManager::getAutosaveFilename() const
     const QString baseDir = defaultSaveLocation();
     const QDir baseDirPath(baseDir);
     const QString filename = formattedFilename();
-    const QString fullpath =
-        autoIncrementFilename(baseDirPath.filePath(filename), Settings::self()->defaultSaveImageFormat().toLower(), &ExportManager::isFileExists);
+    const QString fullpath = autoIncrementFilename(baseDirPath.filePath(filename),
+                                                   Settings::preferredImageFormat().toLower(),
+                                                   &ExportManager::isFileExists);
 
     const QUrl fileNameUrl = QUrl::fromUserInput(fullpath);
     if (fileNameUrl.isValid()) {
@@ -147,19 +147,33 @@ QUrl ExportManager::getAutosaveFilename() const
     }
 }
 
-QString ExportManager::suggestedVideoFilename(const QString &extension) const
+QUrl ExportManager::tempVideoUrl()
 {
-    const QString baseDir = defaultVideoSaveLocation();
+    const auto format = static_cast<VideoPlatform::Format>(Settings::preferredVideoFormat());
+    auto extension = VideoPlatform::extensionForFormat(format);
+    QString baseDir = defaultVideoSaveLocation();
     const QDir baseDirPath(baseDir);
-    const QString filename = formattedFilename(Settings::self()->saveVideoFormat());
-    const QString fullpath = autoIncrementFilename(baseDirPath.filePath(filename), extension, &ExportManager::isFileExists);
+    const QString filename = formattedFilename(Settings::videoFilenameFormat());
+    QString filepath = autoIncrementFilename(baseDirPath.filePath(filename), extension, &ExportManager::isFileExists);
 
-    const QUrl fileNameUrl = QUrl::fromUserInput(fullpath);
-    if (fileNameUrl.isValid() && fileNameUrl.isLocalFile()) {
-        return fileNameUrl.toLocalFile();
-    } else {
+    auto tempDir = temporaryDir();
+    if (!tempDir) {
         return {};
     }
+
+    if (!baseDir.isEmpty() && !baseDir.endsWith(u'/')) {
+        baseDir += u'/';
+    }
+    filepath = tempDir->path() + u'/' + filepath.right(filepath.size() - baseDir.size());
+    return QUrl::fromLocalFile(filepath);
+}
+
+const QTemporaryDir *ExportManager::temporaryDir()
+{
+    if (!m_tempDir) {
+        m_tempDir = std::make_unique<QTemporaryDir>(QDir::tempPath() + u"/Spectacle.XXXXXX"_s);
+    }
+    return m_tempDir->isValid() ? m_tempDir.get() : nullptr;
 }
 
 QString ExportManager::truncatedFilename(QString const &filename) const
@@ -181,34 +195,36 @@ QString ExportManager::formattedFilename(const QString &nameTemplate) const
     QString title = m_windowTitle;
 
     if (!title.isEmpty()) {
-        title.replace(QLatin1Char('/'), QLatin1String("_")); // POSIX doesn't allow "/" in filenames
+        title.replace(u'/', u'_'); // POSIX doesn't allow "/" in filenames
     } else {
         // Remove '%T' with separators around it
-        const auto wordSymbol = QStringLiteral(R"(\p{L}\p{M}\p{N})");
-        const auto separator = QStringLiteral("([^%1]+)").arg(wordSymbol);
-        const auto re = QRegularExpression(QStringLiteral("(.*?)(%1%T|%T%1)(.*?)").arg(separator));
-        baseName.replace(re, QStringLiteral(R"(\1\5)"));
+        const auto wordSymbol = uR"(\p{L}\p{M}\p{N})"_s;
+        const auto separator = u"([^%1]+)"_s.arg(wordSymbol);
+        const auto re = QRegularExpression(u"(.*?)(%1%T|%T%1)(.*?)"_s.arg(separator));
+        baseName.replace(re, uR"(\1\5)"_s);
     }
 
-    QString result = baseName.replace(QLatin1String("%Y"), timestamp.toString(QStringLiteral("yyyy")))
-                             .replace(QLatin1String("%y"), timestamp.toString(QStringLiteral("yy")))
-                             .replace(QLatin1String("%M"), timestamp.toString(QStringLiteral("MM")))
-                             .replace(QLatin1String("%D"), timestamp.toString(QStringLiteral("dd")))
-                             .replace(QLatin1String("%H"), timestamp.toString(QStringLiteral("hh")))
-                             .replace(QLatin1String("%m"), timestamp.toString(QStringLiteral("mm")))
-                             .replace(QLatin1String("%S"), timestamp.toString(QStringLiteral("ss")))
-                             .replace(QLatin1String("%t"), timestamp.toString(QStringLiteral("t")))
-                             .replace(QLatin1String("%T"), title);
+    QString result = baseName.replace("%Y"_L1, timestamp.toString(u"yyyy"_s))
+                             .replace("%y"_L1, timestamp.toString(u"yy"_s))
+                             .replace("%M"_L1, timestamp.toString(u"MM"_s))
+                             .replace("%n"_L1, timestamp.toString(u"MMM"_s))
+                             .replace("%N"_L1, timestamp.toString(u"MMMM"_s))
+                             .replace("%D"_L1, timestamp.toString(u"dd"_s))
+                             .replace("%H"_L1, timestamp.toString(u"hh"_s))
+                             .replace("%m"_L1, timestamp.toString(u"mm"_s))
+                             .replace("%S"_L1, timestamp.toString(u"ss"_s))
+                             .replace("%t"_L1, timestamp.toString(u"t"_s))
+                             .replace("%T"_L1, title);
 
     // check if basename includes %[N]d token for sequential file numbering
     QRegularExpression paddingRE;
-    paddingRE.setPattern(QStringLiteral("%(\\d*)d"));
+    paddingRE.setPattern(u"%(\\d*)d"_s);
     QRegularExpressionMatchIterator it = paddingRE.globalMatch(result);
     if (it.hasNext()) {
         // strip any subdirectories from the template to construct the filename matching regex
         // we are matching filenames only, not paths
-        QString resultCopy = QRegularExpression::escape(result.section(QLatin1Char('/'), -1));
-        QVector<QRegularExpressionMatch> matches;
+        QString resultCopy = QRegularExpression::escape(result.section(u'/', -1));
+        QList<QRegularExpressionMatch> matches;
         while (it.hasNext()) {
             QRegularExpressionMatch paddingMatch = it.next();
             matches.push_back(paddingMatch);
@@ -218,13 +234,13 @@ QString ExportManager::formattedFilename(const QString &nameTemplate) const
                 paddedLength = paddingMatch.captured(1).toInt();
             }
             QString escapedMatch = QRegularExpression::escape(paddingMatch.captured());
-            resultCopy.replace(escapedMatch, QStringLiteral("(\\d{%1,})").arg(QString::number(paddedLength)));
+            resultCopy.replace(escapedMatch, u"(\\d{%1,})"_s.arg(QString::number(paddedLength)));
         }
-        if (result.contains(QLatin1Char('/'))) {
+        if (result.contains(u'/')) {
             // In case the filename template contains a subdirectory,
             // we need to search for files in the subdirectory instead of the baseDir.
             // so let's add that to baseDir before we search for files.
-            baseDir += QStringLiteral("/%1").arg(result.section(QLatin1Char('/'), 0, -2));
+            baseDir += u"/%1"_s.arg(result.section(u'/', 0, -2));
         }
         // search save directory for files
         QDir dir(baseDir);
@@ -254,33 +270,33 @@ QString ExportManager::formattedFilename(const QString &nameTemplate) const
             if (!match.captured(1).isEmpty()) {
                 paddedLength = match.captured(1).toInt();
             }
-            const QString nextFileNumberPadded = QString::number(highestFileNumber + 1).rightJustified(paddedLength, QLatin1Char('0'));
+            const QString nextFileNumberPadded = QString::number(highestFileNumber + 1).rightJustified(paddedLength, u'0');
             result.replace(match.captured(), nextFileNumberPadded);
         }
     }
 
     // Remove leading and trailing '/'
-    while (result.startsWith(QLatin1Char('/'))) {
+    while (result.startsWith(u'/')) {
         result.remove(0, 1);
     }
-    while (result.endsWith(QLatin1Char('/'))) {
+    while (result.endsWith(u'/')) {
         result.chop(1);
     }
 
     if (result.isEmpty()) {
-        result = QStringLiteral("Screenshot");
+        result = u"Screenshot"_s;
     }
     return truncatedFilename(result);
 }
 
 QString ExportManager::autoIncrementFilename(const QString &baseName, const QString &extension, FileNameAlreadyUsedCheck isFileNameUsed) const
 {
-    QString result = truncatedFilename(baseName) + QLatin1String(".") + extension;
+    QString result = truncatedFilename(baseName) + u'.' + extension;
     if (!((this->*isFileNameUsed)(QUrl::fromUserInput(result)))) {
         return result;
     }
 
-    QString fileNameFmt = truncatedFilename(baseName) + QStringLiteral("-%1.");
+    QString fileNameFmt = truncatedFilename(baseName) + u"-%1.";
     for (quint64 i = 1; i < std::numeric_limits<quint64>::max(); i++) {
         result = fileNameFmt.arg(i) + extension;
         if (!((this->*isFileNameUsed)(QUrl::fromUserInput(result)))) {
@@ -291,7 +307,7 @@ QString ExportManager::autoIncrementFilename(const QString &baseName, const QStr
     // unlikely this will ever happen, but just in case we've run
     // out of numbers
 
-    result = fileNameFmt.arg(QLatin1String("OVERFLOW-") + QString::number(QRandomGenerator::global()->bounded(10000)));
+    result = fileNameFmt.arg(u"OVERFLOW-" + QString::number(QRandomGenerator::global()->bounded(10000)));
     return truncatedFilename(result) + extension;
 }
 
@@ -301,7 +317,7 @@ QString ExportManager::imageFileSuffix(const QUrl &url) const
     const QString type = mimedb.mimeTypeForUrl(url).preferredSuffix();
 
     if (type.isEmpty()) {
-        return Settings::self()->defaultSaveImageFormat().toLower();
+        return Settings::self()->preferredImageFormat().toLower();
     }
     return type;
 }
@@ -312,7 +328,7 @@ bool ExportManager::writeImage(QIODevice *device, const QByteArray &suffix)
     // From looking at how QImageWriter handles the built-in supported formats internally,
     // "format" basically means the file extension, not the mimetype.
     QImageWriter imageWriter(device, suffix);
-    imageWriter.setQuality(Settings::self()->compressionQuality());
+    imageWriter.setQuality(Settings::imageCompressionQuality());
     /** Set compression 50 if the format is png. Otherwise if no compression value is specified
      *  it will fallback to using quality (QTBUG-43618) and produce huge files.
      *  See also qpnghandler.cpp#n1075. The other formats that do compression seem to have it
@@ -336,7 +352,7 @@ bool ExportManager::localSave(const QUrl &url, const QString &suffix)
     const QUrl dirPath(url.adjusted(QUrl::RemoveFilename));
     const QDir dir(dirPath.path());
 
-    if (!dir.mkpath(QStringLiteral("."))) {
+    if (!dir.mkpath(u"."_s)) {
         Q_EMIT errorMessage(xi18nc("@info",
                                    "Cannot save screenshot because creating "
                                    "the directory failed:<nl/><filename>%1</filename>",
@@ -405,15 +421,13 @@ QUrl ExportManager::tempSave()
         }
     }
 
-    if (!m_tempDir) {
-        m_tempDir = new QTemporaryDir(QDir::tempPath() + QDir::separator() + QStringLiteral("Spectacle.XXXXXX"));
-    }
-    if (m_tempDir && m_tempDir->isValid()) {
+    auto tempDir = this->temporaryDir();
+    if (tempDir) {
         // create the temporary file itself with normal file name and also unique one for this session
         // supports the use-case of creating multiple screenshots in a row
         // and exporting them to the same destination e.g. via clipboard,
         // where the temp file name is used as filename suggestion
-        const QString baseFileName = m_tempDir->path() + QLatin1Char('/') + QUrl::fromLocalFile(formattedFilename()).fileName();
+        const QString baseFileName = m_tempDir->path() + u'/' + QUrl::fromLocalFile(formattedFilename()).fileName();
 
         QString suffix = imageFileSuffix(QUrl(baseFileName));
         const QString fileName = autoIncrementFilename(baseFileName, suffix, &ExportManager::isTempFileAlreadyUsed);
@@ -460,7 +474,7 @@ bool ExportManager::isFileExists(const QUrl &url) const
         return false;
     }
     // Using StatJob instead of QFileInfo::exists() is necessary for checking non-local URLs.
-    KIO::StatJob *existsJob = KIO::statDetails(url, KIO::StatJob::DestinationSide, KIO::StatNoDetails, KIO::HideProgressInfo);
+    KIO::StatJob *existsJob = KIO::stat(url, KIO::StatJob::DestinationSide, KIO::StatNoDetails, KIO::HideProgressInfo);
 
     existsJob->exec();
 
@@ -496,18 +510,18 @@ void ExportManager::exportImage(ExportManager::Actions actions, QUrl url)
         }
 
         // construct the file name
-        const QString filenameExtension = Settings::self()->defaultSaveImageFormat().toLower();
-        const QString mimetype = QMimeDatabase().mimeTypeForFile(QStringLiteral("~/fakefile.") + filenameExtension, QMimeDatabase::MatchExtension).name();
+        const QString filenameExtension = Settings::self()->preferredImageFormat().toLower();
+        const QString mimetype = QMimeDatabase().mimeTypeForFile(u"~/fakefile."_s + filenameExtension, QMimeDatabase::MatchExtension).name();
         QFileDialog dialog;
         dialog.setAcceptMode(QFileDialog::AcceptSave);
         dialog.setFileMode(QFileDialog::AnyFile);
         QUrl dirUrl = url.adjusted(QUrl::RemoveFilename);
         if (!dirUrl.isValid()) {
-            dirUrl = Settings::self()->lastSaveAsLocation().adjusted(QUrl::RemoveFilename);
+            dirUrl = Settings::self()->lastImageSaveAsLocation().adjusted(QUrl::RemoveFilename);
         }
         dialog.setDirectoryUrl(dirUrl);
-        dialog.selectFile(formattedFilename() + QStringLiteral(".") + filenameExtension);
-        dialog.setDefaultSuffix(QStringLiteral(".") + filenameExtension);
+        dialog.selectFile(formattedFilename() + u"."_s + filenameExtension);
+        dialog.setDefaultSuffix(u"."_s + filenameExtension);
         dialog.setMimeTypeFilters(supportedFilters);
         dialog.selectMimeTypeFilter(mimetype);
 
@@ -537,7 +551,7 @@ void ExportManager::exportImage(ExportManager::Actions actions, QUrl url)
         // "x-kde-force-image-copy" is handled by Klipper.
         // It ensures that the image is copied to Klipper even with the
         // "Non-text selection: Never save in history" setting selected in Klipper.
-        data->setData(QStringLiteral("x-kde-force-image-copy"), QByteArray());
+        data->setData(u"x-kde-force-image-copy"_s, QByteArray());
         KSystemClipboard::instance()->setMimeData(data, QClipboard::Clipboard);
         success = true;
     }
@@ -551,7 +565,7 @@ void ExportManager::exportImage(ExportManager::Actions actions, QUrl url)
             if (m_imageSavedNotInTemp) {
                 // The image has been saved (manually or automatically),
                 // we need to choose that file path
-                url = Settings::self()->lastSaveLocation();
+                url = Settings::self()->lastImageSaveLocation();
             } else {
                 // use a temporary save path, and copy that to clipboard instead
                 url = ExportManager::instance()->tempSave();
@@ -570,6 +584,117 @@ void ExportManager::exportImage(ExportManager::Actions actions, QUrl url)
     }
 }
 
+void ExportManager::exportVideo(ExportManager::Actions actions, const QUrl &inputUrl, QUrl outputUrl)
+{
+    // input can be empty or nonexistent, but not if we're saving
+    const auto &inputFile = inputUrl.toLocalFile();
+    const auto &inputName = inputUrl.fileName();
+    if ((inputName.isEmpty() || !QFileInfo::exists(inputFile)) && actions & (Save | SaveAs)) {
+        Q_EMIT errorMessage(i18nc("@info:shell","Failed to export video: Temporary file URL must be an existing local file"));
+        return;
+    }
+
+    // output can be empty, but not invalid or with an empty name when not empty and saving
+    const auto &outputName = outputUrl.fileName();
+    if (!outputUrl.isEmpty() && (!outputUrl.isValid() || outputName.isEmpty()) && actions & (Save | SaveAs)) {
+        Q_EMIT errorMessage(i18nc("@info:shell","Failed to export video: Output file URL must be a valid URL with a file name"));
+        return;
+    }
+
+    if (actions & SaveAs) {
+        QMimeDatabase mimeDatabase;
+        // construct the file name
+        const auto &extension = inputName.mid(inputName.lastIndexOf(u'.'));
+        const auto &mimetype = mimeDatabase.mimeTypeForFile(inputName, QMimeDatabase::MatchExtension).name();
+        QFileDialog dialog;
+        dialog.setAcceptMode(QFileDialog::AcceptSave);
+        dialog.setFileMode(QFileDialog::AnyFile);
+        const auto outputDir = outputUrl.adjusted(QUrl::RemoveFilename);
+        if (!outputDir.isValid()) {
+            dialog.setDirectoryUrl(Settings::self()->lastVideoSaveAsLocation().adjusted(QUrl::RemoveFilename));
+        } else {
+            dialog.setDirectoryUrl(outputUrl);
+        }
+        dialog.setDefaultSuffix(extension);
+        dialog.selectFile(!outputName.isEmpty() ? outputName : inputName);
+        dialog.setMimeTypeFilters({mimetype});
+        dialog.selectMimeTypeFilter(mimetype);
+
+        // launch the dialog
+        const bool accepted = dialog.exec() == QDialog::Accepted;
+        const auto &selectedUrl = dialog.selectedUrls().value(0, QUrl());
+        if (accepted && !selectedUrl.fileName().isEmpty()) {
+            outputUrl = selectedUrl;
+        }
+        actions.setFlag(SaveAs, accepted && outputUrl.isValid());
+    }
+
+    bool inputFromTemp = temporaryDir() && inputFile.startsWith(m_tempDir->path());
+    if (!outputUrl.isValid()) {
+        if (actions & AnySave && inputFromTemp) {
+            // Use the temp url without the temp dir as the new url, if necessary
+            const auto &tempDirPath = m_tempDir->path() + u'/';
+            const auto &reducedPath = inputFile.right(inputFile.size() - tempDirPath.size());
+            outputUrl = Settings::videoSaveLocation().adjusted(QUrl::StripTrailingSlash);
+            outputUrl.setPath(outputUrl.path() + u'/' + reducedPath);
+        } else {
+            outputUrl = inputUrl;
+        }
+    }
+
+    // When the input is the output, it should still count as a successful save
+    bool saved = inputUrl == outputUrl;
+    if (actions & AnySave && !saved) {
+        const auto &saveDirUrl = outputUrl.adjusted(QUrl::RemoveFilename);
+        bool saveDirExists = false;
+        if (saveDirUrl.isLocalFile()) {
+            saveDirExists = QFileInfo::exists(saveDirUrl.toLocalFile());
+        } else {
+            KIO::ListJob *listJob = KIO::listDir(saveDirUrl);
+            listJob->exec();
+            saveDirExists = listJob->error() == KJob::NoError;
+        }
+        if (!saveDirExists) {
+            KIO::MkpathJob *mkpathJob = KIO::mkpath(saveDirUrl);
+            mkpathJob->exec();
+            saveDirExists = mkpathJob->error() == KJob::NoError;
+        }
+        if (saveDirExists) {
+            if (inputFromTemp) {
+                auto fileMoveJob = KIO::file_move(inputUrl, outputUrl);
+                fileMoveJob->exec();
+                saved = fileMoveJob->error() == KJob::NoError;
+            } else {
+                auto fileCopyJob = KIO::file_copy(inputUrl, outputUrl);
+                fileCopyJob->exec();
+                saved = fileCopyJob->error() == KJob::NoError;
+            }
+        }
+        if (!saved) {
+            actions.setFlag(AnySave, false);
+            Q_EMIT errorMessage(i18nc("@info", "Unable to save recording. Could not move file to location: %1", outputUrl.toString()));
+            return;
+        }
+    }
+
+    bool copiedPath = false;
+    if (actions & CopyPath
+        // This behavior has no relation to the setting in the config UI,
+        // but it was added to solve this feature request:
+        // https://bugs.kde.org/show_bug.cgi?id=357423
+        || (saved && Settings::clipboardGroup() == Settings::PostScreenshotCopyLocation)) {
+        // will be deleted for us by the platform's clipboard manager.
+        auto data = new QMimeData();
+        data->setText(outputUrl.isLocalFile() ? outputUrl.toLocalFile() : outputUrl.toString());
+        KSystemClipboard::instance()->setMimeData(data, QClipboard::Clipboard);
+        copiedPath = true;
+    }
+
+    if (saved || copiedPath) {
+        Q_EMIT videoExported(actions, outputUrl);
+    }
+}
+
 void ExportManager::doPrint(QPrinter *printer)
 {
     QPainter painter;
@@ -591,17 +716,19 @@ void ExportManager::doPrint(QPrinter *printer)
 }
 
 const QMap<QString, KLocalizedString> ExportManager::filenamePlaceholders{
-    {QStringLiteral("%Y"), ki18nc("A placeholder in the user configurable filename will replaced by the specified value", "Year (4 digit)")},
-    {QStringLiteral("%y"), ki18nc("A placeholder in the user configurable filename will replaced by the specified value", "Year (2 digit)")},
-    {QStringLiteral("%M"), ki18nc("A placeholder in the user configurable filename will replaced by the specified value", "Month")},
-    {QStringLiteral("%D"), ki18nc("A placeholder in the user configurable filename will replaced by the specified value", "Day")},
-    {QStringLiteral("%H"), ki18nc("A placeholder in the user configurable filename will replaced by the specified value", "Hour")},
-    {QStringLiteral("%m"), ki18nc("A placeholder in the user configurable filename will replaced by the specified value", "Minute")},
-    {QStringLiteral("%S"), ki18nc("A placeholder in the user configurable filename will replaced by the specified value", "Second")},
-    {QStringLiteral("%t"), ki18nc("A placeholder in the user configurable filename will replaced by the specified value", "Timezone")},
-    {QStringLiteral("%T"), ki18nc("A placeholder in the user configurable filename will replaced by the specified value", "Window Title")},
-    {QStringLiteral("%d"), ki18nc("A placeholder in the user configurable filename will replaced by the specified value", "Sequential numbering")},
-    {QStringLiteral("%Nd"),
+    {u"%Y"_s, ki18nc("A placeholder in the user configurable filename will replaced by the specified value", "Year (4 digit)")},
+    {u"%y"_s, ki18nc("A placeholder in the user configurable filename will replaced by the specified value", "Year (2 digit)")},
+    {u"%M"_s, ki18nc("A placeholder in the user configurable filename will replaced by the specified value", "Month")},
+    {u"%n"_s, ki18nc("A placeholder in the user configurable filename will replaced by the specified value", "Month (localized short name)")},
+    {u"%N"_s, ki18nc("A placeholder in the user configurable filename will replaced by the specified value", "Month (localized long name)")},
+    {u"%D"_s, ki18nc("A placeholder in the user configurable filename will replaced by the specified value", "Day")},
+    {u"%H"_s, ki18nc("A placeholder in the user configurable filename will replaced by the specified value", "Hour")},
+    {u"%m"_s, ki18nc("A placeholder in the user configurable filename will replaced by the specified value", "Minute")},
+    {u"%S"_s, ki18nc("A placeholder in the user configurable filename will replaced by the specified value", "Second")},
+    {u"%t"_s, ki18nc("A placeholder in the user configurable filename will replaced by the specified value", "Timezone")},
+    {u"%T"_s, ki18nc("A placeholder in the user configurable filename will replaced by the specified value", "Window Title")},
+    {u"%d"_s, ki18nc("A placeholder in the user configurable filename will replaced by the specified value", "Sequential numbering")},
+    {u"%Nd"_s,
      ki18nc("A placeholder in the user configurable filename will replaced by the specified value", "Sequential numbering, padded out to N digits")},
 };
 
