@@ -36,12 +36,14 @@ private:
 
 public:
     enum Action {
-        NoActions = 0b0000,
-        Save      = 0b0001,
-        SaveAs    = 0b0010,
-        CopyImage = 0b0100,
-        CopyPath  = 0b1000,
-        AnySave   = Save | SaveAs,
+        NoActions   = 0b00000,
+        Save        = 0b00001,
+        SaveAs      = 0b00010,
+        CopyImage   = 0b00100,
+        CopyPath    = 0b01000,
+        UserAction  = 0b10000,
+        AnySave     = Save | SaveAs,
+        AnyAction   = AnySave | CopyImage | CopyPath,
     };
     Q_DECLARE_FLAGS(Actions, Action)
     Q_FLAG(Action)
@@ -63,9 +65,14 @@ public:
     /**
      * Returns a formatted filename using a template string.
      */
-    QString formattedFilename(const QString &nameTemplate = Settings::saveFilenameFormat()) const;
+    QString formattedFilename(const QString &nameTemplate = Settings::imageFilenameFormat()) const;
 
-    QString suggestedVideoFilename(const QString &extension) const;
+    /**
+     * The URL to record a video with before it is exported.
+     */
+    QUrl tempVideoUrl();
+
+    const QTemporaryDir *temporaryDir();
 
     static const QMap<QString, KLocalizedString> filenamePlaceholders;
 
@@ -74,6 +81,7 @@ Q_SIGNALS:
 
     void errorMessage(const QString &str);
     void imageExported(const ExportManager::Actions &actions, const QUrl &url = {});
+    void videoExported(const ExportManager::Actions &actions, const QUrl &url = {});
 
 public Q_SLOTS:
 
@@ -82,6 +90,7 @@ public Q_SLOTS:
 
     void setWindowTitle(const QString &windowTitle);
     void exportImage(ExportManager::Actions actions, QUrl url = {});
+    void exportVideo(ExportManager::Actions actions, const QUrl &inputUrl, QUrl outputUrl = {});
     void doPrint(QPrinter *printer);
 
 private:
@@ -99,7 +108,7 @@ private:
     QImage m_saveImage;
     QDateTime m_timestamp;
     QUrl m_tempFile;
-    QTemporaryDir *m_tempDir = nullptr;
+    std::unique_ptr<QTemporaryDir> m_tempDir;
     QList<QUrl> m_usedTempFileNames;
     QString m_windowTitle;
 };
