@@ -5,7 +5,6 @@
 #pragma once
 
 #include <QObject>
-#include <QQmlEngine>
 
 #include <memory>
 
@@ -35,11 +34,8 @@ class QSizeF;
 class Geometry : public QObject
 {
     Q_OBJECT
-    QML_ELEMENT
-    QML_SINGLETON
-
 public:
-    explicit Geometry(QObject *parent = nullptr);
+    static Geometry *instance();
 
     /**
      * This returns the logical size of a raw device pixel based on the given device pixel ratio.
@@ -53,24 +49,6 @@ public:
      */
     [[nodiscard]] Q_INVOKABLE static qreal dprRound(qreal value, qreal dpr);
 
-    /**
-     * This rounds a point to a point that should be aligned to hardware pixels
-     * if the given device pixel ratio is correct.
-     */
-    [[nodiscard]] Q_INVOKABLE static QPointF dprRound(QPointF point, qreal dpr);
-
-    /**
-     * This ceils a logical axis value to a value that should be aligned to hardware pixels
-     * if the given device pixel ratio is correct.
-     */
-    [[nodiscard]] Q_INVOKABLE static qreal dprCeil(qreal value, qreal dpr);
-
-    /**
-     * This floors a logical axis value to a value that should be aligned to hardware pixels
-     * if the given device pixel ratio is correct.
-     */
-    [[nodiscard]] Q_INVOKABLE static qreal dprFloor(qreal value, qreal dpr);
-
     /**
      * This converts an X11 screen axis value to a logical screen axis value.
      * Otherwise, it returns the value as it already was.
@@ -129,9 +107,6 @@ public:
      */
     [[nodiscard]] Q_INVOKABLE static QSize rawSize(const QSizeF &size, qreal dpr);
 
-    [[nodiscard]] Q_INVOKABLE static QRectF rectNormalized(const QRectF &rect);
-    [[nodiscard]] Q_INVOKABLE static QRectF rectNormalized(qreal x, qreal y, qreal w, qreal h);
-
     /// Get the rectangle with adjusted left, top, right and bottom sides.
     [[nodiscard]] Q_INVOKABLE static QRectF rectAdjusted(const QRectF &rect, qreal xp1, qreal yp1, qreal xp2, qreal yp2);
 
@@ -155,8 +130,6 @@ public:
      */
     [[nodiscard]] Q_INVOKABLE static QRectF rectBounded(const QRectF &rect, const QRectF &boundsRect, //
                                                         Qt::Orientations orientations = Qt::Horizontal | Qt::Vertical);
-    [[nodiscard]] Q_INVOKABLE static QRectF rectBounded(qreal x, qreal y, qreal width, qreal height, const QRectF &boundsRect, //
-                                                        Qt::Orientations orientations = Qt::Horizontal | Qt::Vertical);
 
     /**
      * Clip the rect to the clipRect on the given axes.
@@ -200,6 +173,7 @@ public:
     [[nodiscard]] Q_INVOKABLE static bool isNull(const QRectF &rect);
 
 private:
+    explicit Geometry(QObject *parent = nullptr);
     Geometry(const Geometry &) = delete;
     Geometry(Geometry &&) = delete;
     Geometry &operator=(const Geometry &) = delete;
