@@ -6,7 +6,8 @@
 
 #include "AnnotationDocument.h"
 #include "EditAction.h"
-#include "Utils.h"
+#include "EffectUtils.h"
+#include "Geometry.h"
 #include "settings.h"
 
 #include <QGuiApplication>
@@ -18,6 +19,8 @@
 #include <cmath>
 #include <memory>
 
+using G = Geometry;
+
 static AnnotationTool::Options optionsForType(AnnotationDocument::EditActionType type)
 {
     switch (type) {
@@ -824,7 +827,6 @@ AnnotationDocument::~AnnotationDocument()
     m_undoStack.clear();
     qDeleteAll(m_redoStack);
     m_redoStack.clear();
-    clearImages();
 }
 
 AnnotationTool *AnnotationDocument::tool() const
@@ -840,13 +842,16 @@ SelectedActionWrapper *AnnotationDocument::selectedActionWrapper() const
 void AnnotationDocument::clear()
 {
     clearAnnotations();
-    clearImages();
+    setImage({});
 }
 
 void AnnotationDocument::cropCanvas(const QRectF &cropRect)
 {
-    QVector<EditAction *> filteredUndo;
-    QVector<EditAction *> filteredRedo;
+    if (cropRect == m_canvasRect) {
+        return;
+    }
+
+    QList<EditAction *> filteredUndo;
     for (auto *ea : std::as_const(m_undoStack)) {
         if (ea->visualGeometry().intersects(cropRect)) {
             ea->translate(-cropRect.topLeft());
@@ -860,6 +865,7 @@ void AnnotationDocument::cropCanvas(const QRectF &cropRect)
     }
     m_undoStack = filteredUndo;
 
+    QList<EditAction *> filteredRedo;
     for (auto *ea : std::as_const(m_redoStack)) {
         if (ea->visualGeometry().intersects(cropRect)) {
             ea->translate(-cropRect.topLeft());
@@ -870,11 +876,7 @@ void AnnotationDocument::cropCanvas(const QRectF &cropRect)
     }
     m_redoStack = filteredRedo;
 
-    for (auto &img : m_canvasImages) {
-        img.rect.setTopLeft(img.rect.topLeft() - cropRect.topLeft());
-    }
-
-    m_canvasSize = cropRect.size();
+    setImage(m_image.copy(G::rectScaled(cropRect, imageDpr()).toAlignedRect()));
 
     Q_EMIT canvasSizeChanged();
     Q_EMIT undoStackDepthChanged();
@@ -884,35 +886,28 @@ void AnnotationDocument::cropCanvas(const QRectF &cropRect)
 
 QSizeF AnnotationDocument::canvasSize() const
 {
-    return m_canvasSize;
+    return m_canvasRect.size();
 }
 
-void AnnotationDocument::addImage(const CanvasImage &canvasImage)
+void AnnotationDocument::setImage(const QImage &image)
 {
-    m_canvasImages << canvasImage;
-
-    QRectF rect;
-    for (const auto &img : qAsConst(m_canvasImages)) {
-        rect = rect.united(img.rect);
-    }
-    m_canvasSize = rect.size();
+    m_image = image;
+    m_canvasRect = {{0, 0}, QSizeF(image.size()) / image.devicePixelRatio()};
 
     Q_EMIT canvasSizeChanged();
+    Q_EMIT imageSizeChanged();
+    Q_EMIT imageDprChanged();
     Q_EMIT repaintNeeded();
 }
 
-QVector<CanvasImage> AnnotationDocument::canvasImages() const
+QSizeF AnnotationDocument::imageSize() const
 {
-    return m_canvasImages;
+    return m_image.size();
 }
 
-void AnnotationDocument::clearImages()
+qreal AnnotationDocument::imageDpr() const
 {
-    m_canvasImages.clear();
-    m_canvasImages.squeeze();
-    m_canvasSize = QSizeF();
-    Q_EMIT canvasSizeChanged();
-    Q_EMIT repaintNeeded();
+    return m_image.devicePixelRatio();
 }
 
 void AnnotationDocument::clearAnnotations()
@@ -931,35 +926,26 @@ void AnnotationDocument::clearAnnotations()
     Q_EMIT repaintNeeded();
 }
 
-inline QPointF zoomPoint(const QPointF &pos, qreal zoomFactor)
-{
-    return QPointF(pos.x() * zoomFactor, pos.y() * zoomFactor);
-}
-
-inline QSizeF zoomSize(const QSizeF &size, qreal zoomFactor)
-{
-    return QSizeF(size.width() * zoomFactor, size.height() * zoomFactor);
-}
-
-inline QRectF zoomRect(const QRectF &rect, qreal zoomFactor)
-{
-    return QRectF(rect.x() * zoomFactor, rect.y() * zoomFactor, rect.width() * zoomFactor, rect.height() * zoomFactor);
-}
-
-void AnnotationDocument::paint(QPainter *painter, const QRectF &viewPort, qreal zoomFactor) const
+void AnnotationDocument::paint(QPainter *painter, const QRectF &viewPort, qreal zoomFactor, RenderOptions options) const
 {
     static QList<EditAction *> stopAtAction = QList<EditAction *>();
     const qreal scale = painter->transform().m11();
 
-    for (const auto &img : qAsConst(m_canvasImages)) {
-        if (viewPort.intersects(img.rect)) {
+    if (options.testFlag(RenderOption::Images)) {
+        auto imageRect = G::rectScaled(viewPort, imageDpr() / zoomFactor);
+        if (zoomFactor == 1) {
+            painter->drawImage({0, 0}, m_image, imageRect);
+        } else {
             // More High quality scale down
-            auto scaledImg = img.image.scaled(zoomSize(img.image.size(), zoomFactor).toSize(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
-            scaledImg.setDevicePixelRatio(img.image.devicePixelRatio());
-            painter->drawImage(zoomPoint((img.rect.topLeft() - viewPort.topLeft()) * img.image.devicePixelRatio(), zoomFactor).toPoint(), scaledImg);
+            auto scaledImg = m_image.scaled(m_image.size() * zoomFactor,
+                                            Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
+            painter->drawImage({0, 0}, scaledImg, imageRect);
         }
     }
 
+    if (!options.testFlag(RenderOption::Annotations)) {
+        return;
+    }
     painter->scale(zoomFactor, zoomFactor);
     painter->setRenderHints({QPainter::Antialiasing, QPainter::TextAntialiasing});
     for (auto *ea : m_undoStack) {
@@ -968,7 +954,7 @@ void AnnotationDocument::paint(QPainter *painter, const QRectF &viewPort, qreal
             return;
         }
 
-        if (!isActionVisible(ea, zoomRect(viewPort, 1 / zoomFactor))) {
+        if (!isActionVisible(ea, G::rectScaled(viewPort, 1 / zoomFactor))) {
             continue;
         }
 
@@ -1054,18 +1040,18 @@ void AnnotationDocument::paint(QPainter *painter, const QRectF &viewPort, qreal
         case Blur: {
             auto *sa = static_cast<ShapeAction *>(ea);
             const QRectF &targetRect = sa->geometry().normalized().translated(-viewPort.topLeft());
-            const qreal factor = 2 * qGuiApp->devicePixelRatio(); // take the maximum scale factor of any screen
-            const qreal dpr = 1.0 / factor;
+            const qreal factor = 2 * imageDpr();
+            const qreal scale = 1.0 / factor;
             if (sa->backingStoreCache().isNull() || sa->backingStoreCache().devicePixelRatio() != painter->deviceTransform().m11()) {
                 stopAtAction << ea;
-                sa->backingStoreCache() = renderToImage(QRectF(QPointF(0, 0), canvasSize()), dpr);
+                sa->backingStoreCache() = renderToImage(QRectF(QPointF(0, 0), canvasSize()), scale);
                 sa->backingStoreCache().setDevicePixelRatio(painter->deviceTransform().m11());
                 stopAtAction.pop_back();
                 // With more scaling, blur more
                 sa->backingStoreCache() = fastPseudoBlur(sa->backingStoreCache(), 4, painter->deviceTransform().m11());
             }
             painter->setRenderHint(QPainter::SmoothPixmapTransform, true);
-            painter->drawImage(targetRect, sa->backingStoreCache(), zoomRect(sa->geometry(), dpr).normalized());
+            painter->drawImage(targetRect, sa->backingStoreCache(), G::rectScaled(sa->geometry(), scale * imageDpr()).normalized());
             painter->setRenderHint(QPainter::SmoothPixmapTransform, false);
             break;
         }
@@ -1073,15 +1059,15 @@ void AnnotationDocument::paint(QPainter *painter, const QRectF &viewPort, qreal
             auto *sa = static_cast<ShapeAction *>(ea);
             const QRectF &targetRect = sa->geometry().normalized().translated(-viewPort.topLeft());
             const qreal factor = 4;
-            const qreal dpr = 1.0 / factor;
+            const qreal scale = 1.0 / factor;
             if (sa->backingStoreCache().isNull()) {
                 stopAtAction << ea;
-                sa->backingStoreCache() = renderToImage(QRectF(QPointF(0, 0), canvasSize()), dpr);
+                sa->backingStoreCache() = renderToImage(QRectF(QPointF(0, 0), canvasSize()), scale);
 
                 stopAtAction.pop_back();
             }
             painter->setRenderHint(QPainter::SmoothPixmapTransform, false);
-            painter->drawImage(targetRect, sa->backingStoreCache(), zoomRect(sa->geometry(), dpr).normalized());
+            painter->drawImage(targetRect, sa->backingStoreCache(), G::rectScaled(sa->geometry(), scale * imageDpr()).normalized());
             break;
         }
         case Text: {
@@ -1110,21 +1096,27 @@ void AnnotationDocument::paint(QPainter *painter, const QRectF &viewPort, qreal
     painter->scale(scale, scale);
 }
 
-QImage AnnotationDocument::renderToImage(const QRectF &viewPort, qreal devicePixelRatio) const
+QImage AnnotationDocument::renderToImage(const QRectF &viewPort, qreal scale, RenderOptions options) const
 {
-    QImage img(viewPort.width() /* devicePixelRatio*/, viewPort.height() /* devicePixelRatio*/, QImage::Format_ARGB32_Premultiplied);
+    QImage img((viewPort.size() * imageDpr()).toSize(), QImage::Format_ARGB32_Premultiplied);
+    img.setDevicePixelRatio(imageDpr());
     img.fill(Qt::transparent);
     QPainter p(&img);
     p.setRenderHint(QPainter::Antialiasing);
     // Makes pixelate and blur look better
     p.setRenderHint(QPainter::SmoothPixmapTransform, true);
-    p.scale(devicePixelRatio, devicePixelRatio);
-    paint(&p, viewPort);
+    p.scale(scale, scale);
+    paint(&p, viewPort, 1, options);
     p.end();
 
     return img;
 }
 
+QImage AnnotationDocument::renderToImage() const
+{
+    return renderToImage(m_canvasRect);
+}
+
 int AnnotationDocument::undoStackDepth() const
 {
     return m_undoStack.count();
