@@ -6,8 +6,6 @@
 
 #pragma once
 
-#include "CanvasImage.h"
-
 #include <QColor>
 #include <QFont>
 #include <QImage>
@@ -36,10 +34,12 @@ class AnnotationDocument : public QObject
     Q_PROPERTY(int redoStackDepth READ redoStackDepth NOTIFY redoStackDepthChanged)
     Q_PROPERTY(int undoStackDepth READ undoStackDepth NOTIFY undoStackDepthChanged)
     Q_PROPERTY(QSizeF canvasSize READ canvasSize NOTIFY canvasSizeChanged)
+    Q_PROPERTY(QSizeF imageSize READ imageSize NOTIFY imageSizeChanged)
+    Q_PROPERTY(qreal imageDpr READ imageDpr NOTIFY imageDprChanged)
 
 public:
     enum EditActionType { None, FreeHand, Highlight, Line, Arrow, Rectangle, Ellipse, Blur, Pixelate, Text, Number, ChangeAction };
-    Q_ENUM(EditActionType);
+    Q_ENUM(EditActionType)
 
     enum class ContinueOption {
         NoOptions    = 0b00,
@@ -49,6 +49,15 @@ public:
     Q_DECLARE_FLAGS(ContinueOptions, ContinueOption)
     Q_FLAG(ContinueOption)
 
+    enum class RenderOption {
+        // No RenderNone because that's pointless
+        Images      = 0b01,
+        Annotations = 0b10,
+        RenderAll   = Images | Annotations
+    };
+    Q_DECLARE_FLAGS(RenderOptions, RenderOption)
+    Q_FLAG(RenderOption)
+
     explicit AnnotationDocument(QObject *parent = nullptr);
     ~AnnotationDocument();
 
@@ -58,20 +67,26 @@ public:
     int undoStackDepth() const;
     int redoStackDepth() const;
 
-    void paint(QPainter *painter, const QRectF &viewPort, qreal zoomFactor = 1.0) const;
-    QImage renderToImage(const QRectF &viewPort, qreal devicePixelRatio) const;
+    void paint(QPainter *painter, const QRectF &viewPort, qreal zoomFactor = 1.0, RenderOptions options = RenderOption::RenderAll) const;
+    QImage renderToImage(const QRectF &viewPort, qreal scale = 1, RenderOptions options = RenderOption::RenderAll) const;
+    QImage renderToImage() const;
 
     // Actions that can't be undone
     void clear();
     void cropCanvas(const QRectF &cropRect);
     QSizeF canvasSize() const;
 
-    void addImage(const CanvasImage &canvasImage);
-    QVector<CanvasImage> canvasImages() const;
-    void clearImages();
+    void setImage(const QImage &image);
 
     void clearAnnotations();
 
+    /**
+     * Image size in raw pixels
+     */
+    QSizeF imageSize() const;
+
+    qreal imageDpr() const;
+
     // True when there is an edit action in the undo stack and it is invalid.
     Q_INVOKABLE bool isLastActionInvalid() const;
 
@@ -96,6 +111,8 @@ Q_SIGNALS:
     void undoStackDepthChanged();
     void redoStackDepthChanged();
     void canvasSizeChanged();
+    void imageSizeChanged();
+    void imageDprChanged();
 
     void repaintNeeded(const QRectF &area = {});
 
@@ -112,10 +129,10 @@ private:
     AnnotationTool *m_tool;
     SelectedActionWrapper *m_selectedActionWrapper;
 
-    QSizeF m_canvasSize;
-    QVector<EditAction *> m_undoStack;
-    QVector<EditAction *> m_redoStack;
-    QVector<CanvasImage> m_canvasImages;
+    QRectF m_canvasRect;
+    QImage m_image;
+    QList<EditAction *> m_undoStack;
+    QList<EditAction *> m_redoStack;
 };
 
 /**
@@ -295,4 +312,5 @@ private:
 QDebug operator<<(QDebug debug, const SelectedActionWrapper *saw);
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(AnnotationDocument::ContinueOptions)
+Q_DECLARE_OPERATORS_FOR_FLAGS(AnnotationDocument::RenderOptions)
 Q_DECLARE_OPERATORS_FOR_FLAGS(AnnotationTool::Options)
