@@ -8,14 +8,15 @@
 #include "CaptureWindow.h"
 
 #include "Config.h"
-#include "Gui/SpectacleWindow.h"
 #include "SpectacleCore.h"
 #include "Gui/SelectionEditor.h"
 #include "spectacle_gui_debug.h"
 
 #include <QScreen>
 
-QVector<CaptureWindow *> CaptureWindow::s_captureWindowInstances = {};
+using namespace Qt::StringLiterals;
+
+QList<CaptureWindow *> CaptureWindow::s_captureWindowInstances = {};
 
 CaptureWindow::CaptureWindow(Mode mode, QScreen *screen, QQmlEngine *engine, QWindow *parent)
     : SpectacleWindow(engine, parent)
@@ -24,6 +25,8 @@ CaptureWindow::CaptureWindow(Mode mode, QScreen *screen, QQmlEngine *engine, QWi
     s_captureWindowInstances.append(this);
     s_isAnnotating = true;
 
+    m_context->setContextObject(this); // Must be before QML is initialized
+
     setFlags({
         Qt::Window, // the default window flag
         Qt::FramelessWindowHint,
@@ -35,29 +38,12 @@ CaptureWindow::CaptureWindow(Mode mode, QScreen *screen, QQmlEngine *engine, QWi
 
     this->setColor(Qt::transparent);
 
-    // setup selectionEditor
-    auto selectionEditor = SelectionEditor::instance();
-    connect(selectionEditor, &SelectionEditor::screensRectChanged, this, [this]() {
-        syncGeometryWithScreen();
-    });
-    connect(this, &CaptureWindow::devicePixelRatioChanged,
-            selectionEditor, &SelectionEditor::devicePixelRatioChanged);
-
-    // set up QML
-    setMode(mode); // sets source and other stuff based on mode.
-    if (auto rootItem = rootObject()) {
-        rootItem->installEventFilter(selectionEditor);
-    }
-
     // follow a screen
     connect(screen, &QScreen::geometryChanged, this, &CaptureWindow::syncGeometryWithScreen);
-    connect(screen, &QScreen::physicalDotsPerInchChanged, this, [this]() {
-        Q_EMIT devicePixelRatioChanged(m_screenToFollow->devicePixelRatio());
-        syncGeometryWithScreen();
-    });
+    connect(screen, &QScreen::physicalDotsPerInchChanged,
+            this, &CaptureWindow::syncGeometryWithScreen);
     syncGeometryWithScreen();
     Q_EMIT screenToFollowChanged();
-    Q_EMIT devicePixelRatioChanged(m_screenToFollow->devicePixelRatio());
 
     // sync visibility
     connect(this, &QWindow::visibilityChanged, this, [this](QWindow::Visibility visibility){
@@ -73,6 +59,18 @@ CaptureWindow::CaptureWindow(Mode mode, QScreen *screen, QQmlEngine *engine, QWi
         }
         s_synchronizingVisibility = false;
     });
+
+    // setup selectionEditor
+    auto selectionEditor = SelectionEditor::instance();
+    connect(selectionEditor, &SelectionEditor::screensRectChanged, this, [this]() {
+        syncGeometryWithScreen();
+    });
+
+    // set up QML
+    setMode(mode); // sets source and other stuff based on mode.
+    if (auto rootItem = rootObject()) {
+        rootItem->installEventFilter(selectionEditor);
+    }
 }
 
 CaptureWindow::~CaptureWindow()
@@ -91,7 +89,7 @@ CaptureWindow::UniquePointer CaptureWindow::makeUnique(Mode mode, QScreen *scree
     });
 }
 
-QVector<CaptureWindow *> CaptureWindow::instances()
+QList<CaptureWindow *> CaptureWindow::instances()
 {
     return s_captureWindowInstances;
 }
@@ -108,11 +106,19 @@ void CaptureWindow::setMode(CaptureWindow::Mode mode)
         QVariantMap initialProperties = {
             // Set the parent in initialProperties to avoid having
             // the parent and window be null in Component.onCompleted
-            {QStringLiteral("parent"), QVariant::fromValue(contentItem())}
+            {u"parent"_s, QVariant::fromValue(contentItem())}
         };
-        setSource(QUrl(QStringLiteral("qrc:/src/Gui/ImageCaptureOverlay.qml")), initialProperties);
+        setSource(QUrl("%1/Gui/ImageCaptureOverlay.qml"_L1.arg(SPECTACLE_QML_PATH)),
+                  initialProperties);
     } else if (mode == Video) {
-        
+        syncGeometryWithScreen();
+        QVariantMap initialProperties = {
+            // Set the parent in initialProperties to avoid having
+            // the parent and window be null in Component.onCompleted
+            {u"parent"_s, QVariant::fromValue(contentItem())}
+        };
+        setSource(QUrl("%1/Gui/VideoCaptureOverlay.qml"_L1.arg(SPECTACLE_QML_PATH)),
+                  initialProperties);
     }
 }
 
