@@ -6,8 +6,8 @@
 
 #include "SelectionEditor.h"
 
-#include "Annotations/AnnotationDocument.h"
-#include "SpectacleCore.h"
+#include "Selection.h"
+#include "Geometry.h"
 #include "settings.h"
 #include "spectacle_gui_debug.h"
 
@@ -20,11 +20,15 @@
 #include <QPainterPath>
 #include <QPalette>
 #include <QPixmapCache>
+#include <QQuickItem>
 #include <QQuickWindow>
 #include <QScreen>
 #include <QtMath>
 #include <qnamespace.h>
 
+using namespace Qt::StringLiterals;
+using G = Geometry;
+
 class SelectionEditorSingleton
 {
 public:
@@ -33,27 +37,21 @@ public:
 
 Q_GLOBAL_STATIC(SelectionEditorSingleton, privateSelectionEditorSelf)
 
-static constexpr int s_handleRadiusMouse = 9;
-static constexpr int s_handleRadiusTouch = 12;
-static constexpr qreal s_increaseDragAreaFactor = 2.0;
-static constexpr int s_minSpacingBetweenHandles = 20;
-static constexpr int s_borderDragAreaSize = 10;
+static constexpr qreal s_handleRadiusMouse = 9;
+static constexpr qreal s_handleRadiusTouch = 12;
+static constexpr qreal s_minSpacingBetweenHandles = 20;
+static constexpr qreal s_magnifierLargeStep = 15;
 
-static constexpr int s_magnifierLargeStep = 15;
-
-static constexpr inline bool isPointInsideCircle(const QPointF &circleCenter, qreal radius, const QPointF &point) noexcept
+// Map the global position of the scene to a logical global position (if necessary),
+// then translate the scene position of the event to a logical global position using
+// the logical global position of the scene. If you try to get the global event position
+// and then convert it to a logical global position, you will get the wrong mouse position
+// since the event position in the scene is already scaled by the scene's device pixel ratio.
+QPointF mapSceneToLogicalGlobalPoint(const QPointF &point, QQuickItem *item)
 {
-    return (std::pow(point.x() - circleCenter.x(), 2) + std::pow(point.y() - circleCenter.y(), 2) <= std::pow(radius, 2)) ? true : false;
-}
-
-static constexpr inline bool inRange(qreal low, qreal high, qreal value) noexcept
-{
-    return value >= low && value <= high;
-}
-
-static constexpr inline bool withinThreshold(qreal offset, qreal threshold) noexcept
-{
-    return std::fabs(offset) <= threshold;
+    auto window = item ? item->window() : nullptr;
+    Q_ASSERT(window != nullptr);
+    return point + G::mapFromPlatformPoint(window->position(), window->devicePixelRatio());
 }
 
 // SelectionEditorPrivate =====================
@@ -67,13 +65,9 @@ public:
 
     SelectionEditor *const q;
 
-    void updateDevicePixelRatio();
     void updateHandlePositions();
 
-    int boundsLeft(int newTopLeftX, const bool mouse = true);
-    int boundsRight(int newTopLeftX, const bool mouse = true);
-    int boundsUp(int newTopLeftY, const bool mouse = true);
-    int boundsDown(int newTopLeftY, const bool mouse = true);
+    qreal dprRound(qreal value) const;
 
     void handleArrowKey(QKeyEvent *event);
 
@@ -85,22 +79,18 @@ public:
     QPointF startPos;
     QPointF initialTopLeft;
     MouseLocation dragLocation = MouseLocation::None;
-    QImage image;
-    QVector<CanvasImage> screenImages;
     qreal devicePixelRatio = 1;
-    qreal devicePixelRatioI = 1;
+    qreal devicePixel = 1;
     QPointF mousePos;
     bool magnifierAllowed = false;
     bool toggleMagnifier = false;
     bool disableArrowKeys = false;
-    QRect screensRect;
+    QRectF screensRect;
     // Midpoints of handles
-    QVector<QPointF> handlePositions = QVector<QPointF>{8};
+    QList<QPointF> handlePositions = QList<QPointF>{8};
     QRectF handlesRect;
     // Radius of handles is either handleRadiusMouse or handleRadiusTouch
     qreal handleRadius = s_handleRadiusMouse;
-    qreal penWidth = 1;
-    qreal penOffset = 0.5;
 };
 
 SelectionEditorPrivate::SelectionEditorPrivate(SelectionEditor *q)
@@ -109,19 +99,6 @@ SelectionEditorPrivate::SelectionEditorPrivate(SelectionEditor *q)
 {
 }
 
-void SelectionEditorPrivate::updateDevicePixelRatio()
-{
-    if (KWindowSystem::isPlatformWayland()) {
-        devicePixelRatio = 1.0;
-    } else {
-        devicePixelRatio = qApp->devicePixelRatio();
-    }
-
-    devicePixelRatioI = 1.0 / devicePixelRatio;
-    penWidth = q->dprRound(1.0);
-    penOffset = penWidth / 2.0;
-}
-
 void SelectionEditorPrivate::updateHandlePositions()
 {
     // Rectangular region
@@ -145,15 +122,15 @@ void SelectionEditorPrivate::updateHandlePositions()
     if (minEdgeLength < minDragHandleSpace) {
         offset = (minDragHandleSpace - minEdgeLength) / 2.0;
     } else {
-        const QRect translatedScreensRect = screensRect.translated(-screensRect.topLeft());
+        const auto translatedScreensRect = screensRect.translated(-screensRect.topLeft());
 
         offsetTop = top - translatedScreensRect.top() - handleRadius;
         offsetTop = (offsetTop >= 0) ? 0 : offsetTop;
 
-        offsetRight = translatedScreensRect.right() - right - handleRadius + penWidth;
+        offsetRight = translatedScreensRect.right() - right - handleRadius + devicePixel;
         offsetRight = (offsetRight >= 0) ? 0 : offsetRight;
 
-        offsetBottom = translatedScreensRect.bottom() - bottom - handleRadius + penWidth;
+        offsetBottom = translatedScreensRect.bottom() - bottom - handleRadius + devicePixel;
         offsetBottom = (offsetBottom >= 0) ? 0 : offsetBottom;
 
         offsetLeft = left - translatedScreensRect.left() - handleRadius;
@@ -187,59 +164,9 @@ void SelectionEditorPrivate::updateHandlePositions()
     Q_EMIT q->handlesRectChanged();
 }
 
-int SelectionEditorPrivate::boundsLeft(int newTopLeftX, const bool mouse)
+qreal SelectionEditorPrivate::dprRound(qreal value) const
 {
-    if (newTopLeftX < 0) {
-        if (mouse) {
-            // tweak startPos to prevent rectangle from getting stuck
-            startPos.setX(startPos.x() + newTopLeftX * devicePixelRatioI);
-        }
-        newTopLeftX = 0;
-    }
-
-    return newTopLeftX;
-}
-
-int SelectionEditorPrivate::boundsRight(int newTopLeftX, const bool mouse)
-{
-    // the max X coordinate of the top left point
-    const int realMaxX = qRound((q->width() - selection->width()) * devicePixelRatio);
-    const int xOffset = newTopLeftX - realMaxX;
-    if (xOffset > 0) {
-        if (mouse) {
-            startPos.setX(startPos.x() + xOffset * devicePixelRatioI);
-        }
-        newTopLeftX = realMaxX;
-    }
-
-    return newTopLeftX;
-}
-
-int SelectionEditorPrivate::boundsUp(int newTopLeftY, const bool mouse)
-{
-    if (newTopLeftY < 0) {
-        if (mouse) {
-            startPos.setY(startPos.y() + newTopLeftY * devicePixelRatioI);
-        }
-        newTopLeftY = 0;
-    }
-
-    return newTopLeftY;
-}
-
-int SelectionEditorPrivate::boundsDown(int newTopLeftY, const bool mouse)
-{
-    // the max Y coordinate of the top left point
-    const int realMaxY = qRound((q->height() - selection->height()) * devicePixelRatio);
-    const int yOffset = newTopLeftY - realMaxY;
-    if (yOffset > 0) {
-        if (mouse) {
-            startPos.setY(startPos.y() + yOffset * devicePixelRatioI);
-        }
-        newTopLeftY = realMaxY;
-    }
-
-    return newTopLeftY;
+    return G::dprRound(value, devicePixelRatio);
 }
 
 void SelectionEditorPrivate::handleArrowKey(QKeyEvent *event)
@@ -250,41 +177,36 @@ void SelectionEditorPrivate::handleArrowKey(QKeyEvent *event)
 
     const auto key = static_cast<Qt::Key>(event->key());
     const auto modifiers = event->modifiers();
-    const qreal step = (modifiers & Qt::ShiftModifier ? 1 : s_magnifierLargeStep);
+    const bool modifySize = modifiers & Qt::AltModifier;
+    const qreal step = modifiers & Qt::ShiftModifier ? devicePixel : dprRound(s_magnifierLargeStep);
     QRectF selectionRect = selection->rectF();
 
     if (key == Qt::Key_Left) {
-        const int newPos = boundsLeft(qRound(selectionRect.left() * devicePixelRatio - step), false);
-        if (modifiers & Qt::AltModifier) {
-            selectionRect.setRight(devicePixelRatioI * newPos + selectionRect.width());
-            selectionRect = selectionRect.normalized();
+        if (modifySize) {
+            selectionRect = G::rectAdjustedVisually(selectionRect, 0, 0, -step, 0);
         } else {
-            selectionRect.moveLeft(devicePixelRatioI * newPos);
+            selectionRect.translate(-step, 0);
         }
     } else if (key == Qt::Key_Right) {
-        const int newPos = boundsRight(qRound(selectionRect.left() * devicePixelRatio + step), false);
-        if (modifiers & Qt::AltModifier) {
-            selectionRect.setRight(devicePixelRatioI * newPos + selectionRect.width());
+        if (modifySize) {
+            selectionRect = G::rectAdjustedVisually(selectionRect, 0, 0, step, 0);
         } else {
-            selectionRect.moveLeft(devicePixelRatioI * newPos);
+            selectionRect.translate(step, 0);
         }
     } else if (key == Qt::Key_Up) {
-        const int newPos = boundsUp(qRound(selectionRect.top() * devicePixelRatio - step), false);
-        if (modifiers & Qt::AltModifier) {
-            selectionRect.setBottom(devicePixelRatioI * newPos + selectionRect.height());
-            selectionRect = selectionRect.normalized();
+        if (modifySize) {
+            selectionRect = G::rectAdjustedVisually(selectionRect, 0, 0, 0, -step);
         } else {
-            selectionRect.moveTop(devicePixelRatioI * newPos);
+            selectionRect.translate(0, -step);
         }
     } else if (key == Qt::Key_Down) {
-        const int newPos = boundsDown(qRound(selectionRect.top() * devicePixelRatio + step), false);
-        if (modifiers & Qt::AltModifier) {
-            selectionRect.setBottom(devicePixelRatioI * newPos + selectionRect.height());
+        if (modifySize) {
+            selectionRect = G::rectAdjustedVisually(selectionRect, 0, 0, 0, step);
         } else {
-            selectionRect.moveTop(devicePixelRatioI * newPos);
+            selectionRect.translate(0, step);
         }
     }
-    selection->setRect(selectionRect);
+    selection->setRect(modifySize ? selectionRect : G::rectBounded(selectionRect, screensRect));
 }
 
 // TODO: change cursor with pointerhandlers in qml?
@@ -308,51 +230,49 @@ void SelectionEditorPrivate::setMouseCursor(QQuickItem *item, const QPointF &pos
 
 SelectionEditor::MouseLocation SelectionEditorPrivate::mouseLocation(const QPointF &pos) const
 {
-    if (isPointInsideCircle(handlePositions[0], handleRadius * s_increaseDragAreaFactor, pos)) {
+    QRectF handleRect(-handleRadius, -handleRadius, handleRadius * 2, handleRadius * 2);
+    if (G::ellipseContains(handleRect.translated(handlePositions[0]), pos)) {
         return MouseLocation::TopLeft;
     }
-    if (isPointInsideCircle(handlePositions[1], handleRadius * s_increaseDragAreaFactor, pos)) {
+    if (G::ellipseContains(handleRect.translated(handlePositions[1]), pos)) {
         return MouseLocation::TopRight;
     }
-    if (isPointInsideCircle(handlePositions[2], handleRadius * s_increaseDragAreaFactor, pos)) {
+    if (G::ellipseContains(handleRect.translated(handlePositions[2]), pos)) {
         return MouseLocation::BottomRight;
     }
-    if (isPointInsideCircle(handlePositions[3], handleRadius * s_increaseDragAreaFactor, pos)) {
+    if (G::ellipseContains(handleRect.translated(handlePositions[3]), pos)) {
         return MouseLocation::BottomLeft;
     }
-    if (isPointInsideCircle(handlePositions[4], handleRadius * s_increaseDragAreaFactor, pos)) {
+    if (G::ellipseContains(handleRect.translated(handlePositions[4]), pos)) {
         return MouseLocation::Top;
     }
-    if (isPointInsideCircle(handlePositions[5], handleRadius * s_increaseDragAreaFactor, pos)) {
+    if (G::ellipseContains(handleRect.translated(handlePositions[5]), pos)) {
         return MouseLocation::Right;
     }
-    if (isPointInsideCircle(handlePositions[6], handleRadius * s_increaseDragAreaFactor, pos)) {
+    if (G::ellipseContains(handleRect.translated(handlePositions[6]), pos)) {
         return MouseLocation::Bottom;
     }
-    if (isPointInsideCircle(handlePositions[7], handleRadius * s_increaseDragAreaFactor, pos)) {
+    if (G::ellipseContains(handleRect.translated(handlePositions[7]), pos)) {
         return MouseLocation::Left;
     }
 
+    const auto rect = selection->normalized();
     // Rectangle can be resized when border is dragged, if it's big enough
-    if (selection->width() >= 100 && selection->height() >= 100) {
-        if (inRange(selection->x(), selection->x() + selection->width(), pos.x())) {
-            if (withinThreshold(pos.y() - selection->y(), s_borderDragAreaSize)) {
-                return MouseLocation::Top;
-            }
-            if (withinThreshold(pos.y() - selection->y() - selection->height(), s_borderDragAreaSize)) {
-                return MouseLocation::Bottom;
-            }
+    if (rect.width() >= 100 && rect.height() >= 100) {
+        if (rect.adjusted(0, -handleRadius, 0, -rect.height() + handleRadius).contains(pos)) {
+            return MouseLocation::Top;
         }
-        if (inRange(selection->y(), selection->y() + selection->height(), pos.y())) {
-            if (withinThreshold(pos.x() - selection->x(), s_borderDragAreaSize)) {
-                return MouseLocation::Left;
-            }
-            if (withinThreshold(pos.x() - selection->x() - selection->width(), s_borderDragAreaSize)) {
-                return MouseLocation::Right;
-            }
+        if (rect.adjusted(0, rect.height() - handleRadius, 0, handleRadius).contains(pos)) {
+            return MouseLocation::Bottom;
+        }
+        if (rect.adjusted(-handleRadius, 0, -rect.width() + handleRadius, 0).contains(pos)) {
+            return MouseLocation::Left;
+        }
+        if (rect.adjusted(rect.width() - handleRadius, 0, handleRadius, 0).contains(pos)) {
+            return MouseLocation::Right;
         }
     }
-    if (selection->contains(pos.toPoint())) {
+    if (rect.contains(pos)) {
         return MouseLocation::Inside;
     }
     return MouseLocation::Outside;
@@ -364,24 +284,11 @@ SelectionEditor::SelectionEditor(QObject *parent)
     : QObject(parent)
     , d(new SelectionEditorPrivate(this))
 {
-    setObjectName(QStringLiteral("selectionEditor"));
-
-    d->updateDevicePixelRatio();
+    setObjectName(u"selectionEditor"_s);
 
     connect(d->selection.get(), &Selection::rectChanged, this, [this](){
         d->updateHandlePositions();
     });
-
-    if (Settings::rememberLastRectangularRegion() == Settings::Always) {
-        QRectF cropRegion = Settings::cropRegion();
-        if (!cropRegion.isEmpty()) {
-            cropRegion.setRect(cropRegion.x() * d->devicePixelRatioI,
-                               cropRegion.y() * d->devicePixelRatioI,
-                               cropRegion.width() * d->devicePixelRatioI,
-                               cropRegion.height() * d->devicePixelRatioI);
-            d->selection->setRect(cropRegion.intersected(QRectF(d->screensRect.x(), d->screensRect.y(), d->screensRect.width(), d->screensRect.height())));
-        }
-    }
 }
 
 SelectionEditor *SelectionEditor::instance()
@@ -399,17 +306,17 @@ qreal SelectionEditor::devicePixelRatio() const
     return d->devicePixelRatio;
 }
 
-QRect SelectionEditor::screensRect() const
+QRectF SelectionEditor::screensRect() const
 {
     return d->screensRect;
 }
 
-int SelectionEditor::width() const
+qreal SelectionEditor::screensWidth() const
 {
     return d->screensRect.width();
 }
 
-int SelectionEditor::height() const
+qreal SelectionEditor::screensHeight() const
 {
     return d->screensRect.height();
 }
@@ -434,167 +341,50 @@ QPointF SelectionEditor::mousePosition() const
     return d->mousePos;
 }
 
-qreal SelectionEditor::dprRound(qreal value, qreal dpr) const
+bool SelectionEditor::acceptSelection(ExportManager::Actions actions)
 {
-    return std::round(value * dpr) / dpr;
+    if (d->screensRect.isEmpty()) {
+        return false;
+    }
+
+    auto selectionRect = d->selection->normalized();
+    if (Settings::rememberLastRectangularRegion() == Settings::Always) {
+        Settings::setCropRegion(selectionRect.toAlignedRect());
+    }
+
+    if (selectionRect.isEmpty()) {
+        selectionRect = d->screensRect;
+    }
+
+    Q_EMIT accepted(selectionRect, actions);
+    return true;
 }
 
-qreal SelectionEditor::dprRound(qreal value) const
+void SelectionEditor::reset()
 {
-    return dprRound(value, d->devicePixelRatio);
-}
-
-void SelectionEditor::setScreenImages(const QVector<CanvasImage> &screenImages)
-{
-    QVector<QPoint> translatedPoints;
-    QRect screensRect;
-    for (int i = 0; i < screenImages.length(); ++i) {
-        const QPoint &screenPos = screenImages[i].rect.topLeft().toPoint();
-        const QImage &image = screenImages[i].image;
-        const qreal dpr = image.devicePixelRatio();
-
-        QRect virtualScreenRect;
-        if (KWindowSystem::isPlatformX11()) {
-            virtualScreenRect = QRect(screenPos, image.size());
-        } else {
-            // `QSize / qreal` divides the int width and int height by the qreal factor,
-            // then rounds the results to the nearest int.
-            virtualScreenRect = QRect(screenPos, image.size() / dpr);
-        }
-        screensRect = screensRect.united(virtualScreenRect);
-
-        translatedPoints.append(screenPos);
+    qreal dpr = qGuiApp->devicePixelRatio();
+    if (d->devicePixelRatio != dpr) {
+        d->devicePixelRatio = dpr;
+        d->devicePixel = 1 / dpr;
+        Q_EMIT devicePixelRatioChanged();
     }
 
-    d->screenImages = screenImages;
-
-    // compute coordinates after scaling
-    for (int i = 0; i < screenImages.length(); ++i) {
-        const QImage &image = screenImages[i].image;
-        const QPoint &p = screenImages[i].rect.topLeft().toPoint();
-        const QSize &size = image.size();
-        const double dpr = image.devicePixelRatio();
-        if (!qFuzzyCompare(dpr, 1.0)) {
-            // must update all coordinates of next rects
-            int newWidth = size.width();
-            int newHeight = size.height();
-
-            int deltaX = newWidth - (size.width());
-            int deltaY = newHeight - (size.height());
-
-            // for the next size
-            for (int i2 = i; i2 < screenImages.length(); ++i2) {
-                auto point = screenImages[i2].rect.topLeft();
-
-                if (point.x() >= newWidth + p.x() - deltaX) {
-                    translatedPoints[i2].setX(translatedPoints[i2].x() + deltaX);
-                }
-                if (point.y() >= newHeight + p.y() - deltaY) {
-                    translatedPoints[i2].setY(translatedPoints[i2].y() + deltaY);
-                }
-            }
-        }
-    }
-
-    d->image = QImage(screensRect.size(), QImage::Format_ARGB32);
-    d->image.fill(Qt::black);
-    QPainter painter(&d->image);
-    // Don't enable SmoothPixmapTransform, we want crisp graphics.
-    for (int i = 0; i < screenImages.length(); ++i) {
-        // Geometry can have negative coordinates,
-        // so it is necessary to subtract the upper left point,
-        // because coordinates on the widget are counted from 0.
-        QImage image = screenImages[i].image;
-        image.setDevicePixelRatio(1);
-        painter.drawImage(translatedPoints[i] - screensRect.topLeft(), image);
-    }
-
-    if (d->screensRect != screensRect) {
-        d->screensRect = screensRect;
+    auto rect = G::logicalScreensRect();
+    if (d->screensRect != rect) {
+        d->screensRect = rect;
         Q_EMIT screensRectChanged();
     }
 
-    Q_EMIT screenImagesChanged();
-}
-
-QVector<CanvasImage> SelectionEditor::screenImages() const
-{
-    return d->screenImages;
-}
-
-bool SelectionEditor::acceptSelection(ExportManager::Actions actions)
-{
-    if (d->screenImages.isEmpty()) {
-        return false;
-    }
-    if (d->selection->isEmpty()) {
-        d->selection->setRect(d->screensRect);
-    }
-    QRect scaledCropRegion(d->selection->alignedRect(d->devicePixelRatio));
-    if (Settings::rememberLastRectangularRegion() == Settings::Always) {
-        Settings::setCropRegion(scaledCropRegion);
-    }
-
-    auto spectacleCore = SpectacleCore::instance();
-    spectacleCore->annotationDocument()->cropCanvas(d->selection->alignedRect(1));
-
-    if (KWindowSystem::isPlatformX11()) {
-        d->image.setDevicePixelRatio(qGuiApp->devicePixelRatio());
-        if (scaledCropRegion.size() != d->image.size()) {
-            Q_EMIT spectacleCore->grabDone(d->image.copy(scaledCropRegion), actions);
-        } else {
-            Q_EMIT spectacleCore->grabDone(d->image, actions);
+    auto remember = Settings::rememberLastRectangularRegion();
+    if (remember == Settings::Never) {
+        d->selection->setRect({});
+    } else if (remember == Settings::Always) {
+        auto cropRegion = Settings::cropRegion();
+        if (cropRegion.width() < 0 || cropRegion.height() < 0) {
+            cropRegion = {0, 0, 0, 0};
         }
-    } else { // Wayland case
-        // QGuiApplication::devicePixelRatio() is calculated by getting the highest screen DPI
-        qreal maxDpr = qGuiApp->devicePixelRatio();
-
-        QRect selectionRect = d->selection->alignedRect();
-        QSize selectionSize = selectionRect.size();
-        QImage output(selectionSize * maxDpr, QImage::Format_ARGB32);
-        output.fill(Qt::black);
-        QPainter painter(&output);
-        // Don't enable SmoothPixmapTransform, we want crisp graphics
-
-        for (auto it = d->screenImages.constBegin(); it != d->screenImages.constEnd(); ++it) {
-            const QRect &screenRect = it->rect.toRect();
-
-            if (selectionRect.intersects(screenRect)) {
-                const QPoint pos = screenRect.topLeft();
-                const qreal dpr = it->image.devicePixelRatio();
-
-                QRect intersected = screenRect.intersected(selectionRect);
-
-                // converts to screen size & position
-                QRect pixelOnScreenIntersected;
-                pixelOnScreenIntersected.moveTopLeft((intersected.topLeft() - pos) * dpr);
-                pixelOnScreenIntersected.setWidth(intersected.width() * dpr);
-                pixelOnScreenIntersected.setHeight(intersected.height() * dpr);
-
-                QImage screenOutput = it->image.copy(pixelOnScreenIntersected);
-
-                // FIXME: this doesn't seem correct
-                if (intersected.size() == selectionSize) {
-                    // short path when single screen
-                    // keep native screen resolution
-                    // we need to set the pixmap dpr to be able to properly align with annotations
-                    screenOutput.setDevicePixelRatio(dpr);
-                    Q_EMIT spectacleCore->grabDone(screenOutput, actions);
-                    return true;
-                }
-
-                // upscale the image according to max screen dpr, to keep the image not distorted
-                output.setDevicePixelRatio(maxDpr);
-                intersected.moveTopLeft((intersected.topLeft() - selectionRect.topLeft()) * maxDpr);
-                intersected.setSize(intersected.size() * maxDpr);
-                painter.drawImage(intersected, screenOutput);
-            }
-        }
-
-        Q_EMIT spectacleCore->grabDone(output, actions);
+        d->selection->setRect(cropRegion);
     }
-
-    return true;
 }
 
 bool SelectionEditor::eventFilter(QObject *watched, QEvent *event)
@@ -688,7 +478,7 @@ void SelectionEditor::hoverMoveEvent(QQuickItem *item, QHoverEvent *event)
     if (!item->window() || !item->window()->screen()) {
         return;
     }
-    d->mousePos = event->posF() + item->window()->screen()->geometry().topLeft() / (KWindowSystem::isPlatformWayland() ? 1 : devicePixelRatio());
+    d->mousePos = mapSceneToLogicalGlobalPoint(event->scenePosition(), item);
     Q_EMIT mousePositionChanged();
     d->setMouseCursor(item, d->mousePos);
 }
@@ -707,11 +497,11 @@ void SelectionEditor::mousePressEvent(QQuickItem *item, QMouseEvent *event)
 
     if (event->button() & (Qt::LeftButton | Qt::RightButton)) {
         if (event->button() & Qt::RightButton) {
-            d->selection->setRect(QRect());
+            d->selection->setRect({});
         }
         item->setFocus(true);
         const bool wasMagnifierAllowed = d->magnifierAllowed;
-        d->mousePos = event->localPos() + item->window()->screen()->geometry().topLeft() / (KWindowSystem::isPlatformWayland() ? 1 : devicePixelRatio());
+        d->mousePos = mapSceneToLogicalGlobalPoint(event->scenePosition(), item);
         Q_EMIT mousePositionChanged();
         auto newDragLocation = d->mouseLocation(d->mousePos);
         if (d->dragLocation != newDragLocation) {
@@ -764,7 +554,7 @@ void SelectionEditor::mouseMoveEvent(QQuickItem *item, QMouseEvent *event)
         return;
     }
 
-    d->mousePos = event->localPos() + item->window()->screen()->geometry().topLeft() / (KWindowSystem::isPlatformWayland() ? 1 : devicePixelRatio());
+    d->mousePos = mapSceneToLogicalGlobalPoint(event->scenePosition(), item);
     Q_EMIT mousePositionChanged();
     const bool wasMagnifierAllowed = d->magnifierAllowed;
     d->magnifierAllowed = true;
@@ -782,15 +572,15 @@ void SelectionEditor::mouseMoveEvent(QQuickItem *item, QMouseEvent *event)
         const bool afterY = d->mousePos.y() >= d->startPos.y();
         d->selection->setRect(afterX ? d->startPos.x() : d->mousePos.x(),
                               afterY ? d->startPos.y() : d->mousePos.y(),
-                              qAbs(d->mousePos.x() - d->startPos.x()) + (afterX ? d->devicePixelRatioI : 0),
-                              qAbs(d->mousePos.y() - d->startPos.y()) + (afterY ? d->devicePixelRatioI : 0));
+                              qAbs(d->mousePos.x() - d->startPos.x()) + (afterX ? d->devicePixel : 0),
+                              qAbs(d->mousePos.y() - d->startPos.y()) + (afterY ? d->devicePixel : 0));
         break;
     }
     case MouseLocation::Outside: {
         d->selection->setRect(qMin(d->mousePos.x(), d->startPos.x()),
                               qMin(d->mousePos.y(), d->startPos.y()),
-                              qAbs(d->mousePos.x() - d->startPos.x()) + d->devicePixelRatioI,
-                              qAbs(d->mousePos.y() - d->startPos.y()) + d->devicePixelRatioI);
+                              qAbs(d->mousePos.x() - d->startPos.x()) + d->devicePixel,
+                              qAbs(d->mousePos.y() - d->startPos.y()) + d->devicePixel);
         break;
     }
     case MouseLocation::Top:
@@ -799,7 +589,7 @@ void SelectionEditor::mouseMoveEvent(QQuickItem *item, QMouseEvent *event)
         d->selection->setRect(d->selection->x(),
                               afterY ? d->startPos.y() : d->mousePos.y(),
                               d->selection->width(),
-                              qAbs(d->mousePos.y() - d->startPos.y()) + (afterY ? d->devicePixelRatioI : 0));
+                              qAbs(d->mousePos.y() - d->startPos.y()) + (afterY ? d->devicePixel : 0));
         break;
     }
     case MouseLocation::Right:
@@ -807,29 +597,15 @@ void SelectionEditor::mouseMoveEvent(QQuickItem *item, QMouseEvent *event)
         const bool afterX = d->mousePos.x() >= d->startPos.x();
         d->selection->setRect(afterX ? d->startPos.x() : d->mousePos.x(),
                               d->selection->y(),
-                              qAbs(d->mousePos.x() - d->startPos.x()) + (afterX ? d->devicePixelRatioI : 0),
+                              qAbs(d->mousePos.x() - d->startPos.x()) + (afterX ? d->devicePixel : 0),
                               d->selection->height());
         break;
     }
     case MouseLocation::Inside: {
         d->magnifierAllowed = false;
-        // We use some math here to figure out if the diff with which we
-        // move the rectangle with moves it out of bounds,
-        // in which case we adjust the diff to not let that happen
-
-        // new top left point of the rectangle
-        QPointF newTopLeft = (d->mousePos - d->startPos + d->initialTopLeft) * d->devicePixelRatio;
-
-        const QRectF newRect(newTopLeft, d->selection->sizeF() * d->devicePixelRatio);
-
-        const QRectF translatedScreensRect = d->screensRect.translated(-d->screensRect.topLeft());
-        if (!translatedScreensRect.contains(newRect)) {
-            // Keep the item inside the scene screen region bounding rect.
-            newTopLeft.setX(qMin(translatedScreensRect.right() - newRect.width(), qMax(newTopLeft.x(), translatedScreensRect.left())));
-            newTopLeft.setY(qMin(translatedScreensRect.bottom() - newRect.height(), qMax(newTopLeft.y(), translatedScreensRect.top())));
-        }
-
-        d->selection->moveTo(newTopLeft * d->devicePixelRatioI);
+        // We use some math here to figure out if the diff with which we move the rectangle
+        QRectF newRect(d->mousePos - d->startPos + d->initialTopLeft, d->selection->sizeF());
+        d->selection->setRect(G::rectBounded(newRect, d->screensRect));
         break;
     }
     default:
