@@ -7,21 +7,14 @@
 #ifndef SELECTIONEDITOR_H
 #define SELECTIONEDITOR_H
 
-#include "Selection.h"
-#include "CanvasImage.h"
 #include "ExportManager.h"
 
-#include <QMap>
-#include <QPixmap>
-#include <QQuickPaintedItem>
-
 #include <memory>
 
-class ComparableQPoint;
+class QHoverEvent;
 class QKeyEvent;
 class QMouseEvent;
-class QPainter;
-class CaptureWindow;
+class QQuickItem;
 class Selection;
 class SelectionEditorPrivate;
 
@@ -34,7 +27,7 @@ class SelectionEditor : public QObject
     Q_OBJECT
     Q_PROPERTY(Selection *selection READ selection CONSTANT FINAL)
     Q_PROPERTY(qreal devicePixelRatio READ devicePixelRatio NOTIFY devicePixelRatioChanged FINAL)
-    Q_PROPERTY(QRect screensRect READ screensRect NOTIFY screensRectChanged FINAL)
+    Q_PROPERTY(QRectF screensRect READ screensRect NOTIFY screensRectChanged FINAL)
     Q_PROPERTY(MouseLocation dragLocation READ dragLocation NOTIFY dragLocationChanged FINAL)
     Q_PROPERTY(QRectF handlesRect READ handlesRect NOTIFY handlesRectChanged FINAL)
     Q_PROPERTY(bool magnifierAllowed READ magnifierAllowed NOTIFY magnifierAllowedChanged FINAL)
@@ -68,9 +61,10 @@ public:
 
     qreal devicePixelRatio() const;
 
-    QRect screensRect() const;
-    int width() const;
-    int height() const;
+    QRectF screensRect() const;
+
+    qreal screensWidth() const;
+    qreal screensHeight() const;
 
     MouseLocation dragLocation() const;
 
@@ -79,34 +73,20 @@ public:
     bool magnifierAllowed() const;
     QPointF mousePosition() const;
 
-    /**
-     * Round value to be physically pixel perfect, based on the device pixel ratio.
-     * Meant to be used with coordinates, line widths and shape sizes.
-     * This is not meant to be used in QML.
-     */
-    qreal dprRound(qreal value, qreal dpr) const;
-    qreal dprRound(qreal value) const;
-
-    /**
-     * Sets the images to use for each screen.
-     * Each image should match a screen 1:1 in the order that they are listed
-     * by QGuiApplication::screens().
-     * Only used with CaptureModeModel::RectangularRegion
-     */
-    Q_SLOT void setScreenImages(const QVector<CanvasImage> &screenImages);
-    QVector<CanvasImage> screenImages() const;
-
     Q_SLOT bool acceptSelection(ExportManager::Actions actions = {});
 
+    void reset();
+
 Q_SIGNALS:
     void devicePixelRatioChanged();
     void screensRectChanged();
     void dragLocationChanged();
     void handlesRectChanged();
-    void screenImagesChanged();
     void magnifierAllowedChanged();
     void mousePositionChanged();
 
+    void accepted(const QRectF &rect, const ExportManager::Actions &actions);
+
 protected:
     bool eventFilter(QObject *watched, QEvent *event) override;
     void keyPressEvent(QQuickItem *item, QKeyEvent *event);
