@@ -10,31 +10,31 @@
 
 #include "ExportManager.h"
 #include "SpectacleCore.h"
+#include "Geometry.h"
+#include "Gui/ExportMenu.h"
+#include "Gui/HelpMenu.h"
+#include "Gui/OptionsMenu.h"
+#include "Gui/WidgetWindowUtils.h"
 #include "spectacle_gui_debug.h"
 
 #include <KIO/JobUiDelegateFactory>
 #include <KIO/OpenFileManagerWindowJob>
-#include <KIO/OpenUrlJob>
-#include <KUrlMimeData>
 #include <KWindowSystem>
 
-#include <Kirigami/Units>
 #include <QApplication>
 #include <QColorDialog>
-#include <QDrag>
 #include <QFontDialog>
 #include <QtQml>
 #include <utility>
 
 #ifdef XCB_FOUND
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-#include <QX11Info>
-#else
 #include <private/qtx11extras_p.h>
 #endif
-#endif
 
-QVector<SpectacleWindow *> SpectacleWindow::s_spectacleWindowInstances = {};
+using namespace Qt::StringLiterals;
+using G = Geometry;
+
+QList<SpectacleWindow *> SpectacleWindow::s_spectacleWindowInstances = {};
 bool SpectacleWindow::s_synchronizingVisibility = false;
 bool SpectacleWindow::s_synchronizingTitle = false;
 SpectacleWindow::TitlePreset SpectacleWindow::s_lastTitlePreset = Default;
@@ -44,22 +44,10 @@ bool SpectacleWindow::s_isAnnotating = false;
 
 SpectacleWindow::SpectacleWindow(QQmlEngine *engine, QWindow *parent)
     : QQuickView(engine, parent)
-    , m_exportMenu(new ExportMenu)
-    , m_optionsMenu(new OptionsMenu)
-    , m_helpMenu(new HelpMenu)
     , m_context(new QQmlContext(engine->rootContext(), this))
 {
     s_spectacleWindowInstances.append(this);
 
-    if (m_exportMenu->winId()) {
-        m_exportMenu->windowHandle()->setTransientParent(this);
-    }
-    if (m_optionsMenu->winId()) {
-        m_optionsMenu->windowHandle()->setTransientParent(this);
-    }
-    if (m_helpMenu->winId()) {
-        m_helpMenu->windowHandle()->setTransientParent(this);
-    }
     connect(engine, &QQmlEngine::quit, QCoreApplication::instance(), &QCoreApplication::quit, Qt::QueuedConnection);
     connect(this, &QQuickView::statusChanged, this, [](QQuickView::Status status){
         if (status == QQuickView::Error) {
@@ -81,7 +69,7 @@ SpectacleWindow::SpectacleWindow(QQmlEngine *engine, QWindow *parent)
     if (KWindowSystem::isPlatformX11()) {
         // do the xcb shenanigans
         xcb_connection_t *xcbConn = QX11Info::connection();
-        const QByteArray effectName = QByteArrayLiteral("_KDE_NET_WM_SKIP_CLOSE_ANIMATION");
+        const QByteArray effectName = "_KDE_NET_WM_SKIP_CLOSE_ANIMATION"_ba;
 
         xcb_intern_atom_cookie_t atomCookie = xcb_intern_atom_unchecked(xcbConn, false, effectName.length(), effectName.constData());
         QScopedPointer<xcb_intern_atom_reply_t, QScopedPointerPodDeleter> atom(xcb_intern_atom_reply(xcbConn, atomCookie, nullptr));
@@ -96,7 +84,7 @@ SpectacleWindow::SpectacleWindow(QQmlEngine *engine, QWindow *parent)
 
     // set up QML
     setResizeMode(QQuickView::SizeRootObjectToView);
-    m_context->setContextProperty(QStringLiteral("contextWindow"), this);
+    m_context->setContextProperty(u"contextWindow"_s, this);
 }
 
 SpectacleWindow::~SpectacleWindow()
@@ -106,41 +94,12 @@ SpectacleWindow::~SpectacleWindow()
 
 qreal SpectacleWindow::logicalX() const
 {
-#ifdef XCB_FOUND
-    if (KWindowSystem::isPlatformX11()) {
-        return x() / devicePixelRatio();
-    } else
-#endif
-    {
-        return x();
-    }
+    return G::mapFromPlatformValue(x(), devicePixelRatio());
 }
 
 qreal SpectacleWindow::logicalY() const
 {
-#ifdef XCB_FOUND
-    if (KWindowSystem::isPlatformX11()) {
-        return y() / devicePixelRatio();
-    } else
-#endif
-    {
-        return y();
-    }
-}
-
-ExportMenu *SpectacleWindow::exportMenu() const
-{
-    return m_exportMenu.get();
-}
-
-OptionsMenu *SpectacleWindow::optionsMenu() const
-{
-    return m_optionsMenu.get();
-}
-
-HelpMenu *SpectacleWindow::helpMenu() const
-{
-    return m_helpMenu.get();
+    return G::mapFromPlatformValue(y(), devicePixelRatio());
 }
 
 bool SpectacleWindow::isAnnotating() const
@@ -167,7 +126,7 @@ void SpectacleWindow::unminimize()
     setWindowStates(windowStates().setFlag(Qt::WindowMinimized, false));
 }
 
-QVector<SpectacleWindow *> SpectacleWindow::instances()
+QList<SpectacleWindow *> SpectacleWindow::instances()
 {
     return s_spectacleWindowInstances;
 }
@@ -217,7 +176,7 @@ void SpectacleWindow::closeAll()
 
 qreal SpectacleWindow::dprRound(qreal value) const
 {
-    return std::round(value * devicePixelRatio()) / devicePixelRatio();
+    return G::dprRound(value, devicePixelRatio());
 }
 
 QString SpectacleWindow::baseFileName(const QUrl &url) const
@@ -231,11 +190,11 @@ QString SpectacleWindow::titlePresetString(TitlePreset preset, const QString &fi
         return i18ncp("@title:window", "%1 second", "%1 seconds",
                       qCeil(SpectacleCore::instance()->captureTimeRemaining() / 1000.0));
     } else if (preset == TitlePreset::Unsaved) {
-        return i18nc("@title:window Unsaved Screenshot", "Unsaved") + QStringLiteral("*");
+        return i18nc("@title:window Unsaved Screenshot", "Unsaved") + u"*"_s;
     } else if (preset == TitlePreset::Saved && !fileName.isEmpty()) {
         return fileName;
     } else if (preset == TitlePreset::Modified && !fileName.isEmpty()) {
-        return fileName + QStringLiteral("*");
+        return fileName + u"*"_s;
     } else if (preset == TitlePreset::Previous && !s_previousTitle.isEmpty()) {
         return s_previousTitle;
     }
@@ -288,33 +247,48 @@ void SpectacleWindow::setSource(const QUrl &source, const QVariantMap &initialPr
 
 void SpectacleWindow::save()
 {
-    ExportManager::instance()->exportImage(ExportManager::Save, SpectacleCore::instance()->outputUrl());
+    SpectacleCore::instance()->syncExportImage();
+    ExportManager::instance()->exportImage(ExportManager::Save | ExportManager::UserAction,
+                                           SpectacleCore::instance()->outputUrl());
 }
 
 void SpectacleWindow::saveAs()
 {
-    ExportManager::instance()->exportImage(ExportManager::SaveAs);
+    if (SpectacleCore::instance()->videoMode()) {
+        ExportManager::instance()->exportVideo(ExportManager::SaveAs | ExportManager::UserAction,
+                                               SpectacleCore::instance()->currentVideo());
+        return;
+    }
+    SpectacleCore::instance()->syncExportImage();
+    ExportManager::instance()->exportImage(ExportManager::SaveAs | ExportManager::UserAction);
 }
 
 void SpectacleWindow::copyImage()
 {
     SpectacleCore::instance()->syncExportImage();
-    ExportManager::instance()->exportImage(ExportManager::CopyImage);
+    ExportManager::instance()->exportImage(ExportManager::CopyImage | ExportManager::UserAction);
 }
 
 void SpectacleWindow::copyLocation()
 {
-    ExportManager::instance()->exportImage(ExportManager::CopyPath);
+    if (SpectacleCore::instance()->videoMode()) {
+        ExportManager::instance()->exportVideo(ExportManager::CopyPath | ExportManager::UserAction,
+                                               SpectacleCore::instance()->currentVideo());
+        return;
+    }
+    SpectacleCore::instance()->syncExportImage();
+    ExportManager::instance()->exportImage(ExportManager::CopyPath | ExportManager::UserAction);
 }
 
 void SpectacleWindow::showPrintDialog()
 {
-    m_exportMenu->openPrintDialog();
+    SpectacleCore::instance()->syncExportImage();
+    ExportMenu::instance()->openPrintDialog();
 }
 
 void SpectacleWindow::showPreferencesDialog()
 {
-    m_optionsMenu->showPreferencesDialog();
+    OptionsMenu::instance()->showPreferencesDialog();
 }
 
 void SpectacleWindow::showFontDialog()
@@ -333,9 +307,7 @@ void SpectacleWindow::showFontDialog()
     QFontDialog *dialog = new QFontDialog(font);
     dialog->setAttribute(Qt::WA_DeleteOnClose);
 
-    if (dialog->winId()) {
-        dialog->windowHandle()->setTransientParent(this);
-    }
+    setWidgetTransientParent(dialog, this);
 
     if (flags().testFlag(Qt::WindowStaysOnTopHint)) {
         dialog->setWindowFlag(Qt::WindowStaysOnTopHint);
@@ -348,10 +320,10 @@ void SpectacleWindow::showFontDialog()
         // https://bugreports.qt.io/browse/QTBUG-63792
         // https://bugs.kde.org/show_bug.cgi?id=378523
         if (newFont.weight() == QFont::Normal
-            && (newFont.styleName() == QLatin1String("Regular")
-                || newFont.styleName() == QLatin1String("Normal")
-                || newFont.styleName() == QLatin1String("Book")
-                || newFont.styleName() == QLatin1String("Roman"))) {
+            && (newFont.styleName() == "Regular"_L1
+                || newFont.styleName() == "Normal"_L1
+                || newFont.styleName() == "Book"_L1
+                || newFont.styleName() == "Roman"_L1)) {
             newFont.setStyleName(QString());
         }
         auto tool = SpectacleCore::instance()->annotationDocument()->tool();
@@ -418,9 +390,7 @@ void SpectacleWindow::showColorDialog(int option)
     dialog->setAttribute(Qt::WA_DeleteOnClose);
     dialog->setOption(QColorDialog::ShowAlphaChannel);
 
-    if (dialog->winId()) {
-        dialog->windowHandle()->setTransientParent(this);
-    }
+    setWidgetTransientParent(dialog, this);
 
     if (flags().testFlag(Qt::WindowStaysOnTopHint)) {
         dialog->setWindowFlag(Qt::WindowStaysOnTopHint);
@@ -446,61 +416,11 @@ void SpectacleWindow::showColorDialog(int option)
     dialog->open();
 }
 
-void SpectacleWindow::openUrlExternally(const QUrl &url)
-{
-    auto job = new KIO::OpenUrlJob(url);
-    job->setUiDelegate(KIO::createDefaultJobUiDelegate(KJobUiDelegate::AutoHandlingEnabled, nullptr));
-    job->start();
-}
-
 void SpectacleWindow::openContainingFolder(const QUrl &url)
 {
     KIO::highlightInFileManager({url});
 }
 
-void SpectacleWindow::startDrag()
-{
-    auto exportManager = ExportManager::instance();
-    if (exportManager->image().isNull()) {
-        return;
-    }
-
-    const QUrl tempFile = SpectacleCore::instance()->videoMode() ? SpectacleCore::instance()->currentVideo() : exportManager->tempSave();
-    if (!tempFile.isValid()) {
-        return;
-    }
-
-    auto mimeData = new QMimeData;
-    mimeData->setUrls(QList<QUrl>{tempFile});
-    // "application/x-kde-suggestedfilename" is handled by KIO/PasteJob.
-    // It is only used when QMimeData::formats() is empty or when the user is
-    // prompted to set a filename for the content after drag & drop or paste.
-    // When QMimeData::formats() is empty, a dialog for picking the data format
-    // is supposed to appear.
-    // It's likely that users will never see the data format dialog with Spectacle.
-    mimeData->setData(QStringLiteral("application/x-kde-suggestedfilename"), QFile::encodeName(tempFile.fileName()));
-    KUrlMimeData::exportUrlsToPortal(mimeData);
-
-    auto dragHandler = new QDrag(this);
-    dragHandler->setMimeData(mimeData);
-
-    if (SpectacleCore::instance()->videoMode()) {
-        Kirigami::Units units;
-        auto iconSize = units.iconSizes()->large();
-        dragHandler->setPixmap(QIcon::fromTheme(QStringLiteral("video-x-matroska")).pixmap(iconSize, iconSize));
-    } else {
-        QSize size = exportManager->image().size();
-        QPixmap pixmap = QPixmap::fromImage(exportManager->image());
-        // TODO: use the composed pixmap with annotations instead
-        if (size.width() > 256 || size.height() > 256) {
-            dragHandler->setPixmap(pixmap.scaled(256, 256, Qt::KeepAspectRatio, Qt::SmoothTransformation));
-        } else {
-            dragHandler->setPixmap(pixmap);
-        }
-    }
-    dragHandler->exec(Qt::CopyAction);
-}
-
 void SpectacleWindow::mousePressEvent(QMouseEvent *event)
 {
     // QMenus need to be closed by hand when used from QML, see plasma-workspace/shellcorona.cpp
@@ -553,7 +473,9 @@ void SpectacleWindow::keyReleaseEvent(QKeyEvent *event)
         SpectacleCore::instance()->takeNewScreenshot();
     } else if (event->matches(QKeySequence::HelpContents)) {
         event->accept();
-        m_helpMenu->showAppHelp();
+        HelpMenu::instance()->showAppHelp();
     }
     m_pressedKeys = {};
 }
+
+#include "moc_SpectacleWindow.cpp"
