@@ -7,6 +7,8 @@
 #include "VideoPlatform.h"
 #include <QTimerEvent>
 
+using namespace Qt::StringLiterals;
+
 VideoPlatform::VideoPlatform(QObject *parent)
     : QObject(parent)
 {
@@ -45,3 +47,31 @@ void VideoPlatform::timerEvent(QTimerEvent *event)
         Q_EMIT recordedTimeChanged();
     }
 }
+
+QString VideoPlatform::extensionForFormat(Format format)
+{
+    switch (format) {
+    case WebM_VP9: return "webm"_L1;
+    case MP4_H264: return "mp4"_L1;
+    default: return {};
+    }
+}
+
+VideoPlatform::Format VideoPlatform::formatForExtension(const QString &extension)
+{
+    auto lowercaseExtension = extension.toLower();
+    if (lowercaseExtension == "webm"_L1) {
+        return WebM_VP9;
+    } else if (lowercaseExtension == "mp4"_L1) {
+        return MP4_H264;
+    } else {
+        return NoFormat;
+    }
+}
+
+VideoPlatform::Format VideoPlatform::formatForPath(const QString &path)
+{
+    return formatForExtension(path.mid(path.lastIndexOf(u'.') + 1));
+}
+
+#include "moc_VideoPlatform.cpp"
